#****************************************************************************#
#                                                                            #
#               Fonctions d'acces et d'execution d'outils exterieus          #
#                                                                            #
#****************************************************************************#

# Mise a jour $Id: spc_external.tcl 15408 2020-12-03 22:05:13Z bmauclaire $

####################################################################
# Efface le contenu de la Console Audela
#
# Auteur : Benjamin MAUCLAIRE
# Date creation : 20210314
# Date modification : 20210314
####################################################################

proc spc_clearc {} {
   ::console::clear
}
#**********************************************************************************#


####################################################################
# Construit la liste des noms generiques des series dans le repertoire de travail
#
# Auteur : Benjamin MAUCLAIRE
# Date creation : 20120808
# Date modification : 20120808
####################################################################

proc spc_nomsgeneriques {} {
   global audace
   set liste_noms_generiques [ liste_series ]
   set liste_additif [ glob -nocomplain -tail -dir $audace(rep_images) *smd* ]
   if { [ llength $liste_additif ]!=0 } {
      foreach newfile $liste_additif { lappend liste_noms_generiques $newfile }
   }
   set liste_noms_generiques [ lsort -dictionary $liste_noms_generiques ]
   return $liste_noms_generiques
}
#**********************************************************************************#


####################################################################
# Mise à jour d'Spcaudace sous Linux
#
# Auteur : Benjamin MAUCLAIRE
# Date creation : 2020-02-28
# Date modification : 2020-04-04
# Basé sur : https://stackoverflow.com/questions/35673588/how-to-download-a-file-using-http-package
####################################################################

proc spc_upgrade { args } {
   #--- Packages nécessaires :
   global spcaudace audace tcl_platform
   package require http
   #package require tar
   #https://core.tcl-lang.org/tcllib/doc/tcllib-1-18/embedded/www/tcllib/files/modules/zip/decode.html
   if { $tcl_platform(platform) != "windows" } { package require zipfile::decode }

   #--- Paramètres pour powershell :
   set pshell "C:/Windows/SysWOW64/WindowsPowerShell/v1.0/powershell.exe"
   set ps_script_name "script_unzip.ps1"
   set path2script [ file join $::conf(rep_images) $ps_script_name ]
   
   #--- Regle les repertoires et verifie si pkexec est installé :
   #set spc_old "$audace(rep_images)/spcaudace-$spcaudace(num_version)"
   set spc_old "$::env(HOME)/spcaudace-$spcaudace(num_version)"
   set spc_destination "$audace(rep_install)/gui/audace/plugin/tool"
   #set spc_destination "$audace(rep_images)/gui/audace/plugin/tool"
   #set spc_destination "/usr/lib/audela-3.0.0b4/gui/audace/plugin/tool"
   if { [ regexp {home} $spc_destination match resu ] } {
      set flag_local 1
      ::console::affiche_resultat "Upgrade will be in the home directory.\n"
   } else {
      set flag_local 0
      ::console::affiche_resultat "Upgrade will be in the operating system directory.\n"
      if { $tcl_platform(platform)=="unix" && ! [ file exists /usr/bin/pkexec ] } {
         ::console::affiche_erreur "You have to install pkexec (policykit-1 package) in your system. Exit.\n"
         return ""
      } elseif { $tcl_platform(platform)=="windows" } {
         ::console::affiche_prompt "So Audela has to be ran as Superuser\n"
      }
   }


   #--- Recherche la version diponible sur le site d'Spcaudace pour fabriquer le nom du fichier à telecharger :
   set pagecontent [ spc_read_url_contents ${spcaudace(webpage)}/index.php ]
   #set spcaudace(flag_verifversion) 1
   if { $pagecontent!=0 } {
      #-- Récupère le numéro de version de Spcaudace disponible sur le site :
      #- La chaine de caractères détectée est : "KIT : spcaudace-uv.xy"
      #- Elle est mise en commentaire sur dans barnav.php car l'affichage actuel est "KIT version..."
      regexp {KIT\s:\sspcaudace\-([0-9]+\.[0-9]+)} $pagecontent match version_site
      #::console::affiche_resultat "Numéro de la version du site : $version_site\n"
      set spc_name "spcaudace-${version_site}"
   } else {
      ::console::affiche_erreur "Check your Internet connexion.\n"
      return ""
   }

   #--- Télécharge l'archive :
   ::console::affiche_resultat "Downloading archive ${spc_name}.zip. Be patient...\n"
   #::console::affiche_resultat "   (can be up to ~10 min. on Windows with DSL connexion)\n"
   set spc_stats "$spcaudace(webpage)/sources/stata.php?file=$spc_name"
   set spc_source "$spcaudace(webpage)/sources/${spc_name}.zip"
   set spc_archive "$audace(rep_images)/${spc_name}.zip"
   #-- Stats update :
   set token_stats [ ::http::geturl $spc_stats ]
   http::cleanup $token_stats
   #-- Download :
   set spc_archive_id [ open "$spc_archive" wb ]
   fconfigure $spc_archive_id -translation binary
   set token [ ::http::geturl $spc_source -channel $spc_archive_id -binary 1 ]
   if { [ ::http::status $token ] eq "ok" && [ ::http::ncode $token ] == 200 } {
      ::console::affiche_resultat "SpcAudace downloaded successfuly.\n"
   }
   #- Wait 2s before closing:
   sleepy 2000
   http::cleanup $token
   close $spc_archive_id
   #-- IL FAUT DU ZIP pour un dl rapide !
   #set answer [ catch { exec $spcaudace(rep_spcaudace)/plugins/curl.exe $spc_source -s --outputfile $audace(rep_images)/${spc_name}.tgz } ]
   ##powershell.exe -Command "Invoke-WebRequest -OutFile ./file-name https://location/file-name"
   #set answer [ catch { exec powershell.exe -Command "Invoke-WebRequest -OutFile  $audace(rep_images)/${spc_name}.zip $spc_source" } ]

   
   #--- Installation de l'archive :
   ::console::affiche_resultat "Installation of ${spc_name}...\n"
   if { $flag_local==0 && $tcl_platform(platform)=="unix" } { ::console::affiche_prompt "(User's password will be asked two times)\n" }
   #-- Sauvegarde de la version actuelle :
   if { [ file exists "$spc_old" ] } { file delete -force "$spc_old" }
   file copy -force "${spc_destination}/spcaudace" "$spc_old"
   if { $flag_local || $tcl_platform(platform)=="windows" } {
      file delete -force "${spc_destination}/spcaudace"
   } else {
      exec pkexec rm -rf "${spc_destination}/spcaudace"
   }

   #-- Décompression de l'archive :
   if { $tcl_platform(platform)=="windows" } {
      ##set answer [ catch { exec powershell.exe -Command "Expand-Archive -LiteralPath $spc_archive -DestinationPath $spc_destination" } ]
      #- Méthode avec unzip.exe :
      set tarot_unzip [ file join $audace(rep_plugin) tool sn_tarot unzip.exe ]
      set answer [ catch { exec $tarot_unzip -d "$spc_destination" "$spc_archive" } ]
      #- Méthode powershell en admin :
      #Compress-Archive -DestinationPath C:/coucou.zip -Path C:/test -CompressionLevel optimal
      #set ps_script_id [ open "$path2script" w+ ]
      #fconfigure $ps_script_id -translation crlf
      #puts $ps_script_id "Expand-Archive -Path $spc_archive -DestinationPath $spc_destination"
      #close $ps_script_id
      #catch { exec $pshell -Command "& {Start-Process Powershell.exe -Verb RunAs -ArgumentList '-File','$path2script'}" }
      #file delete -force "$path2script"
   } else {
      ::zipfile::decode::unzipfile "$spc_archive" "$spc_destination"
   }
   #if {  $flag_local } {
   #   ::zipfile::decode::unzipfile "$spc_archive" "$spc_destination"
   #} else {
   #   exec pkexec unzip "$spc_archive" -C "$spc_destination/"
   #}
   file delete -force "$spc_archive"
   ::console::affiche_resultat "Old SpcAudace release saved in: $spc_old\n"
   ::console::affiche_prompt "SpcAudace updated successfuly. Audela has to be restarted.\n"
}
#**********************************************************************************#


####################################################################
# Compare la version d'SpcAduace en cours d'execution et averti de la maj
#
# Auteur : Benjamin MAUCLAIRE
# Date creation : 20120201
# Date modification : 20120201
####################################################################

proc spc_versionsite {} {
   global spcaudace flagvs_ok audace

   #--- Teste si la verification a deja ete effectuee :
   if { $spcaudace(flag_verifversion) } { return "" }

   #--- Recherche la version diponible sur le site d'Spcaudace :
   set pagecontent [ spc_read_url_contents ${spcaudace(webpage)}/index.php ]
   set spcaudace(flag_verifversion) 1

   if { $pagecontent!=0 } {
      #--- Debug :
      if { 1==0 } {
         set file_id [open "$audace(rep_images)/webpage.html" w+]
         #-- independamment du systeme LINUX ou WINDOWS
         fconfigure $file_id -translation crlf
         puts $file_id "$pagecontent"
         close $file_id
      }

      #--- Récupère le numéro de version de Spcaudace disponible sur le site :
      #-- La chaine de caractères détectée est : "KIT : spcaudace-uv.xy"
      #-- Elle est mise en commentaire sur dans barnav.php car l'affichage actuel est "KIT version..."
      regexp {KIT\s:\sspcaudace\-([0-9]+\.[0-9]+)} $pagecontent match version_site
      ::console::affiche_resultat "Numéro de la version du site : $version_site\n"
      
      #--- Compare avec la version en cours d'exécution et signale la maj :
      if { $spcaudace(num_version)<$version_site } {
         ::console::affiche_erreur "A newer SpcAudace release is available. You should upgrade your SpcAudace!\n"

         #-- Création de la fenêtre :
         set flagvs_ok 0
         if { [ winfo exists .spcv ] } {
            destroy .spcv
         }
         toplevel .spcv
         wm geometry .spcv 300x50+135+60
         wm title .spcv "A newer SpcAudace release is available"
         wm transient .spcv .audace

         #--- Textes d'avertissement
         label .spcv.lab -text "You should upgrade your SpcAudace!"
         pack .spcv.lab -expand true -expand true -fill both

         #--- Sous-trame pour boutons
         frame .spcv.but
         pack .spcv.but -expand true -fill both

         #--- Bouton "Yes"
         button .spcv.but.1 -command {set flagvs_ok 1} -text "Yes"
         pack .spcv.but.1 -side left -expand true -fill both

         #--- Bouton "No"
         button .spcv.but.2 -command {set flagvs_ok 0} -text "No"
         pack .spcv.but.2 -side left -expand true -fill both

         #--- Focus sur la fenetre d'alerte
         focus .spcv

         #--- Attend le click user
         vwait flagvs_ok

         if { $flagvs_ok == 1 } {
            #--- Ouvre l'outil "Mise a jour d'AudeLA"
            ::confVisu::selectTool 1 ::updateaudela
            #--- Positionne la combobox sur l'adresse web du site SpcAudace
            .audace.tool.updateaudela.frame1.nb.fdownloadPage.address.list.e configure -text "http://spcaudace.free.fr/"
            #--- Charge la page html de mise a jour de SpcAudace
            ::updateaudela::download::connect
            #--- Ferme la fenetre d'alerte
            destroy .spcv
            #--- Ferme la fenetre principale de SpcAudace
            destroy .spc
            #return ""
         } else {
            #--- Focus sur la fenetre principale de SpcAudace
            #- Créé une erreur lorsque la fenêtre Spcaudace n'est pas ouverte !
            #focus .spc
            #--- Ferme la fenetre d'alerte
            destroy .spcv
         }

      } else {
         ::console::affiche_prompt "\nYour SpcAudace release is up to date.\n"
      }
   } else {
      ::console::affiche_resultat "Page récupérée vide.\n"
   }
}
#**********************************************************************************#



####################################################################
# Lecture d'une page HTML via http
#
# Version originale : Alain KLOTZ
# Auteur : Benjamin MAUCLAIRE
# Date creation : 20120201
# Date modification : 20120201
####################################################################

proc spc_read_url_contents { url {fullfile_out ""} } {
   package require http

   if { [ spc_test_url $url ] } {
      set token [::http::geturl $url]
      upvar #0 $token state
      set res $state(body)
      set len [string length $res]
      if {$fullfile_out!=""} {
         set f [open $fullfile_out w]
         puts -nonewline $f "$res"
         close $f
      }
      return $res
   } else {
      return 0
   }
}
#**********************************************************************************#


####################################################################
# Lecture d'une page HTML via http
#
# Auteur : Benjamin MAUCLAIRE
# Date creation : 20120201
# Date modification : 20120201
####################################################################

proc spc_test_url { url } {
   package require http

   if { [ catch { ::http::geturl $url -timeout 2000 } ] } { return 0 } else {  return 1 }
}
#**********************************************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter la documentation d'SpcAudace
# Auteur : Benjamin MAUCLAIRE
# Date création :  28-08-2006
# Date de mise à jour : 28-08-2006
################################################################################

proc spc_help {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
       ::audace::Lance_Site_htm "$spcaudace(sitedoc)"
       if { 1==0 } {
        if { $::tcl_platform(platform) == "windows" } {
           ::audace::Lance_Site_htm [ file join file:///[ file join $::audace(rep_plugin) tool spcaudace doc "$spcaudace(spcdoc)" ] ]

        } else {
            if { [ file exists $spcaudace(spcdoc) ] } {
               set answer [ catch { exec $conf(editsite_htm) $spcaudace(spcdoc) & } ]
           } else {
               set answer [ catch { exec $conf(editsite_htm) $spcaudace(sitedoc) & } ]
           }
        }
       }
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter le site d'SpcAudACE
# Auteur : Benjamin MAUCLAIRE
# Date création :  02-02-2008
# Date de mise à jour : 02-02-2008
################################################################################

proc spc_webpage {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(webpage) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter le site arasbeam
# Auteur : Benjamin MAUCLAIRE
# Date création :  20-09-2008
# Date de mise à jour : 20-09-2008
################################################################################

proc spc_arasbeam {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(sitearasbeam) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter la base BeSS
# Auteur : Benjamin MAUCLAIRE
# Date création :  23-04-2007
# Date de mise à jour : 23-04-2007
################################################################################

proc spc_bess {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(sitebess) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter la base UVES
# Auteur : Benjamin MAUCLAIRE
# Date création :  23-04-2007
# Date de mise à jour : 23-04-2007
################################################################################

proc spc_uves {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(siteuves) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter la base Simbad
# Auteur : Benjamin MAUCLAIRE
# Date création :  24-04-2007
# Date de mise à jour : 24-04-2007
################################################################################

proc spc_simbad {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(sitesimbad) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter la page des surveys
# Auteur : Benjamin MAUCLAIRE
# Date création :  24-04-2007
# Date de mise à jour : 24-04-2007
################################################################################

proc spc_surveys {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(sitesurveys) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du navigateur internet pour consulter la page Be de C. Buil
# Auteur : Benjamin MAUCLAIRE
# Date création :  24-04-2007
# Date de mise à jour : 24-04-2007
################################################################################

proc spc_bebuil {} {

    global spcaudace conf

    if { $conf(editsite_htm)!="" } {
        set answer [ catch { exec $conf(editsite_htm) $spcaudace(sitebebuil) & } ]
    } else {
        ::console::affiche_resultat "Configurer \"Editeurs/Navigateur web\" pour permettre l'affichage de la documentation d'SpcAudACE\n"
    }
}
#*********************************************************#


###############################################################################
# Procédure de lancement du logiciel SPECTRUM
# Auteur : Benjamin MAUCLAIRE
# Date création :  23-04-2007
# Date de mise à jour : 23-04-2007
################################################################################

proc spc_spectrum {} {

    global spcaudace conf tcl_platform

    if { $tcl_platform(os)=="Linux" } {
        #set answer [ catch { exec xterm -r -e $spcaudace(spectrum)/spectrum & } ]
        set answer [ catch { exec $spcaudace(spectrum)/spectrum_sh.tcl & } ]
        ::console::affiche_resultat "$answer\n"
    } else {
        set answer [ catch { exec $spcaudace(spectrum)/setup.exe & } ]
        ::console::affiche_resultat "$answer\n"
    }
}
#*********************************************************#

